/**************************************************************

aterm.c (C-Munipack project)
Copyright (C) 2003 David Motl, dmotl@volny.cz

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

**************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "comfun.h"
#include "convert.h"
#include "cmdparser.h"
#include "amass.h"
#include "aterm.h"

static int    mode = 0;
static double ra  = 0.0;
static double dec = 0.0;
static double lat = 0.0;
static double lon = 0.0;

void PrintCoords(int mode);
void PrintTHelp();

int airmass_term(const char *strra, const char *strdec, const char *strlon, const char *strlat)
{
	char *argv[64], *prompt, outf[MAXLINE], *endptr;
	char datestr[32], timestr[32], azstr[32], altstr[32];
	int argc;
	double jdin, x, az, alt;

	printf("This is terminal mode. Enter 'h' command for help or 'q' for exit.\n\n");
	cmpack_setoutput(NULL, MPK_LEVEL_QUIET); 	// disable outputs

	mode = 0;
 	if (cmpack_strtora(strra, &x)==0) {
	 	ra = x;
 		mode |= 0x01;
	}
	if (cmpack_strtodec(strdec, &x)==0) {
		dec = x;
		mode |= 0x02;
	}
	if (cmpack_strtolon(strlon, &x)==0) {
		lon = x;
		mode |= 0x04;
	}
	if (cmpack_strtolat(strlat, &x)==0) {
		lat = x;
		mode |= 0x08;
	}
		
	PrintCoords(mode);

    while (mode>=0) {
	    if ((mode & 0x01)==0) prompt = "R.A.:";
	    else if ((mode & 0x02)==0) prompt = "DEC.:";
	    else if ((mode & 0x04)==0) prompt = "LON.:";
	    else if ((mode & 0x08)==0) prompt = "LAT.:";
	    else prompt = "JD:";

	    getstr(prompt, outf, MAXLINE);
	    argc = parse_args(argv, outf);
	    if (argc==0) {
		    // No command => Print coordinates
		    PrintCoords(mode);
		    continue;
	    }
  		if (strncmp("q", argv[0], strlen(argv[0])) == 0) {
	  		// Quit command
	    	mode = -1;
	        break;
  		}
  		if ((strncmp("h", argv[0], strlen(argv[0])) == 0)
  		   ||(strncmp("?", argv[0], strlen(argv[0])) == 0)
  		   ||(strncmp("help", argv[0], strlen(argv[0])) == 0)) {
	  		// Print help
	    	PrintTHelp();
			continue;
  		}
  		if (strncmp("ra", argv[0], strlen(argv[0]))==0) {
	  		if (argc==1) {
		  		// Print R.A.
		  		PrintCoords(0x01);
	  		} else {
		  		// Set R.A.
		  	    if (cmpack_strtora(argv[1], &x)==0) {
			  	    ra = x;
			  	    PrintCoords(0x01);
			  	    mode |= 0x01;
		  	    } else {
			  	    PrintError(PROGNAME, NULL, 0, MPK_INVALID_OBJRA);
		  	    }
	  		}
	  		continue;
  		}
  		if (strncmp("dec", argv[0], strlen(argv[0]))==0) {
	  		if (argc==1) {
		  		// Print DEC.
		  		PrintCoords(0x02);
	  		} else {
		  		// Set DEC.
		  	    if (cmpack_strtodec(argv[1], &x)==0) {
			  	    dec = x;
			  	    PrintCoords(0x02);
			  	    mode |= 0x02;
		  	    } else {
			  	    PrintError(PROGNAME, NULL, 0, MPK_INVALID_OBJDEC);
		  	    }
	  		}
	  		continue;
  		}
  		if (strncmp("lon", argv[0], strlen(argv[0]))==0) {
	  		if (argc==1) {
		  		// Print LON.
		  		PrintCoords(0x04);
	  		} else {
		  		// Set LON.
		  	    if (cmpack_strtolon(argv[1], &x)==0) {
			  	    lon = x;
			  	    PrintCoords(0x04);
			  	    mode |= 0x04;
		  	    } else {
			  	    PrintError(PROGNAME, NULL, 0, MPK_INVALID_OBSLON);
		  	    }
	  		}
	  		continue;
  		}
  		if (strncmp("lat", argv[0], strlen(argv[0]))==0) {
	  		if (argc==1) {
		  		// Print LAT.
		  		PrintCoords(0x08);
	  		} else {
		  		// Set LAT.
		  	    if (cmpack_strtolat(argv[1], &x)==0) {
			  	    lat = x;
			  	    PrintCoords(0x08);
			  	    mode |= 0x08;
		  	    } else {
			  	    PrintError(PROGNAME, NULL, 0, MPK_INVALID_OBSLAT);
		  	    }
	  		}
	  		continue;
  		}
  		
  		if ((mode & 0x01)==0) {
	  		// Set R.A.
	  	    if (cmpack_strtora(outf, &x)==0) {
		  	    ra = x;
		  	    PrintCoords(0x01);
		  	    mode |= 0x01;
	  	    } else {
		  	    PrintError(PROGNAME, NULL, 0, MPK_INVALID_OBJRA);
	  	    }
	  		continue;
  		}
  		if ((mode & 0x02)==0) {
	  		// Set DEC.
	  	    if (cmpack_strtodec(outf, &x)==0) {
		  	    dec = x;
		  	    PrintCoords(0x02);
		  	    mode |= 0x02;
	  	    } else {
		  	    PrintError(PROGNAME, NULL, 0, MPK_INVALID_OBJDEC);
	  	    }
	  		continue;
  		}
  		if ((mode & 0x04)==0) {
	  		// Set LON.
	  	    if (cmpack_strtolon(outf, &x)==0) {
		  	    lon = x;
		  	    PrintCoords(0x04);
		  	    mode |= 0x04;
	  	    } else {
		  	    PrintError(PROGNAME, NULL, 0, MPK_INVALID_OBSLON);
	  	    }
	  		continue;
  		}
  		if ((mode & 0x08)==0) {
	  		// Set LAT.
	  	    if (cmpack_strtolat(outf, &x)==0) {
		  	    lat = x;
		  	    PrintCoords(0x08);
		  	    mode |= 0x08;
	  	    } else {
		  	    PrintError(PROGNAME, NULL, 0, MPK_INVALID_OBSLAT);
	  	    }
	  		continue;
  		}
  		
  		jdin = strtod(outf,&endptr);
  		if (endptr!=outf && ((jdin>=10000 && jdin<100000) || (jdin>=2400000 && jdin<3000000))) {
	  		if ((mode & 0x03)!=0x03) {
	  			PrintError(PROGNAME, NULL, 0, MPK_MISSING_OBJCOORDS);
	  			continue;
  			}
	  		if ((mode & 0x0C)!=0x0C) {
	  			PrintError(PROGNAME, NULL, 0, MPK_MISSING_OBSCOORDS);
	  			continue;
  			}
  			if (jdin<100000)
	      		jdin += 2400000.0;
	      	printf("------------------------------------------------\n");
      		cmpack_jdtostr(jdin, datestr, timestr);
      		printf("Julian date: %.*f\n", DECIMALS, jdin);
      		printf("Date & time: %s %s UT\n", datestr, timestr);
      		cmpack_azalt(jdin, ra, dec, lon, lat, &az, &alt); 
      		cmpack_aztostr(az, azstr);
      		cmpack_alttostr(alt, altstr);
      		printf("Az. & alt. : %s, %s\n", azstr, altstr);
	      	x = cmpack_airmass(jdin, ra, dec, lon, lat);
    	  	printf("Airmass (X): %.3f\n", x); 
	      	printf("------------------------------------------------\n");
      		continue;
      }
      printf("Invalid julian date.\n");
  }
  printf("\nExiting terminal mode.\n");

  /* Normal return */
  return 0;
}

void PrintCoords(int mode)
/* Prints objects coordinates */
{
	char buf[32];
	
    if ((mode & 0x01)!=0) {
	  	cmpack_ratostr(ra, buf);
	  	printf("R.A. = %s\n", buf);
  	}
    if ((mode & 0x02)!=0) {
	  	cmpack_dectostr(dec, buf);
	  	printf("DEC. = %s\n", buf);
  	}
    if ((mode & 0x04)!=0) {
	  	cmpack_lontostr(lon, buf);
	  	printf("LON. = %s\n", buf);
  	}
    if ((mode & 0x08)!=0) {
	  	cmpack_lattostr(lat, buf);
	  	printf("LAT. = %s\n", buf);
  	}
  	if (mode==0) {
		printf("Object's coordinates have not been set yet.\n");
	}
}

void PrintTHelp()
// Print help
{
 printf("airmass (C-Munipack) %s\n",VERSION);
 printf(COPYRIGHTLINE);
 printf(WARRANTYLINE);
 printf("\n");
 printf("Commands of terminal mode:\n");
 printf("    2499999.999  -- Julian date to convert\n");
 printf("    q            -- exit program\n");
 printf("    empty line   -- print current settings\n");
 printf("    h, ? or help -- print this help\n");
 printf("    ra hh:mm     -- Set R.A. of object (Use format: hhmm or hh:mm)\n");
 printf("    dec +dd:mm   -- Set DEC. of object (Use format: +ddmm or +dd:mm)\n");
 printf("    lon +dd:mm   -- Set longitude of observer (Use format: +ddmm or +dd:mm)\n");
 printf("    lat +dd:mm   -- Set latitude of observer (Use format: +ddmm or +dd:mm)\n");
 printf("\n");
 PrintCoords(mode);
 printf("\n");
 printf(HOMEPAGELINE);
 printf(BUGREPORTS);
 printf("\n");
}
