/**************************************************************

robmean.c (C-Munipack project)
Copyright (C) 2003 David Motl, dmotl@volny.cz

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

**************************************************************/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <math.h>

#include "comfun.h"
#include "robmean.h"

#define hampel_a   1.7
#define hampel_b   3.4
#define hampel_c   8.5
#define sign(x,y)  ((y)>=0?fabs(x):-fabs(x))
#define fabs(x)    ((x)>=0?(x):-(x))
#define epsilon(x) 0.00000001
#define maxit      50

static double hampel(double x)
{
  if (fabs(x)<hampel_a) {
     return x;
  } else 
  if ((hampel_a<=fabs(x))&&(fabs(x)<hampel_b)) {
     return sign(hampel_a,x);
  } else
  if ((hampel_b<=fabs(x))&&(fabs(x)<hampel_c)) {
     return hampel_a*(x - sign(hampel_c,x))/(hampel_b - hampel_c);
  } else
     return 0.0;
}

static double dhampel(double x)
{
  if (fabs(x)<hampel_a) { /* absolutni hodnota z derivace x */
      return 1.0;
  } else    
  if ((hampel_b<=fabs(x))&&(fabs(x)<hampel_c)) {
     return -sign(hampel_a/(hampel_b - hampel_c),x);
  } else
     return 0.0;
}

//-------------------   SINGLE PRECISION VERSION   ---------------------

static float qmedF(int n, float *a)
/* Vypocet medianu algoritmem Quick Median (Wirth) */
{
	float w, x; 
	int i, j;
	int k = ((n&1)?(n/2):((n/2)-1));
	int l = 0;
	int r = n-1;

    while (l<r) {
       x = a[k];
       i = l;
       j = r;
       do {
          while (a[i]<x) i++;
          while (x<a[j]) j--;
          if (i<=j) {
             w = a[i]; a[i]=a[j]; a[j]=w; 
             i++;
             j--;
          }
       } while (i<=j);   
       if (j<k) l=i;
       if (k<i) r=j;
    }
    return a[k];
}    

static void MADestF(int n, float *x, float *mean, float *stddev, float *median, float *mad) 
/* estimate median and median of absoluted deviations */
{
    int i;
    double d, a = 0.0, s = 0.0; 
    float *xx; 
    
    xx = malloc(n*sizeof(float));
    memcpy(xx,x,n*sizeof(float));

    *median = qmedF(n,xx);
    for (i=0;i<n;i++) {
	    xx[i] = fabs(x[i]-(*median));
	    d = x[i];
        a += d;
        s += d*d;
    }
    *mean = (float)(a/n);
    if ((s-(a*a)/n)>=0) 
    	*stddev = (float)sqrt((s-(a*a)/n)/(n-1));
    else
    	*stddev = 0.0;
    *mad = qmedF(n,xx);
    free(xx);

    /*printf("Median, median of absoluted deviations : %.3f, %.3f\n", *median, *mad); */
}
 
static int am1F(int n, float *x, float *t, float *dt)
/* Newton's iterations */
{
  int i, it;
  float a,b,d,r,s,s2,sum1,sum2,sum3,rp,psir;
  
  if (n<1) {
	*t = 0.0;    /* a few data */
	*dt = -1.0;   
    return 1;   
  }
  
  if (n==1) {  /* only one point, but correct case */
    *t = x[0]; 
    *dt = 0.0;
    return 0; 
  }
  
  /* initial values */
  MADestF(n,x,t,dt,&a,&s);
  
  s = (float)(s/0.6745);
  s2 = s*s;

  /* identical points on input */
  if (fabs(s)<epsilon(s)) {                 
      /*a=0.0; for (i=0;i<n;i++) a+=x[i];
      *t = (a/n);
      *dt= 0.0; */
      /*sprintf(buf, "n = %d, t = %.8f, dt = %.8f", n, *t, *dt);
      PrintOut(1, buf);
      for (i=0; i<n; i++) {
      	sprintf(buf, "x[%d] = %.8f", i, x[i]);
      	PrintOut(1, buf);
  	  } */
      return 0;
  }    
  
  /* corrector's estimation */
  for (it=1;it<=maxit;it++) {
       sum1 = 0.0; sum2 = 0.0; sum3 = 0.0;
       for (i=0;i<n;i++) {
          r = (x[i] - a)/s;
          rp = (float)hampel(r);
          sum1 += rp;
          sum2 += (float)dhampel(r);
          sum3 += rp*rp;
          
          //printf("Here1=%f,%f,%f.\n",sum1,sum2,sum3); 
       }
       if (fabs(sum2)<epsilon(sum2)) {
          /*printf("Here2=%.12f,%.12f.\n",fabs(sum2),a); 
          for (i=0;i<n;i++) {
	          r = (x[i] - a)/s;
              printf("x[%d] = %.4f\n", i, x[i]);
              printf("r[%d] = %.4f\n", i, (x[i] - a)/s);
              printf("hampel[%d] = %.4f\n", i, hampel(r));
              printf("dhampel[%d] = %.4f\n", i, dhampel(r));
          }
          printf("----------------\n");*/
          
          ////////////////// POZOR /////////////////////////////
          *t = a; 
          *dt = -1.0;
          ////////////////// POZOR /////////////////////////////
          return 2;
       }
       d = s*sum1/sum2;
       a = a + d;
       b = s2*n/(n-1)*sum3/(sum2*sum2);

       /* exit of iterations: */
       /*printf(stderr,"Prumer, prirustek, chyba: %f, %f, %f, %d\n",a,d,sqrt(b)); */
       if((it>2)&&((d*d<0.01*b)||(fabs(d)<10.0*epsilon(d)))) break;
  }

  /* deviation's estimation */
  sum1 = 0.0; sum3 = 0.0;
  for (i=0;i<n;i++) {
       r = (x[i]-a)/s;
       psir = (float)hampel(r);
       sum1 += psir*psir;
       sum3 += (float)dhampel(r);
  }

  if (sum3==0.0) {
    //printf("Here3=%.12f,%.12f.\n",sum3,a); 
    ////////////////// POZOR /////////////////////////////
    *t = a;
    *dt = -1.0;
    ////////////////// POZOR /////////////////////////////
    return 3;
  }
  
  *t = a;
  *dt = (float)sqrt(s2*n/(n-1)*sum1/(sum3*sum3)*n);
/*  printf("Normal %.3f, %.3f.\n",a,*dt); */
  
  /* Normal return */
  return 0;
}

void cmpack_robustmean(int n, float *array, float *median, float *sig)
{
  am1F(n,array,median,sig);
}

//----------------------   DOUBLE PRECISION VERSION   -----------------------------

static double qmedD(int n, double *a)
/* Vypocet medianu algoritmem Quick Median (Wirth) */
{
	double w, x; 
	int i, j;
	int k = ((n&1)?(n/2):((n/2)-1));
	int l = 0;
	int r = n-1;

    while (l<r) {
       x = a[k];
       i = l;
       j = r;
       do {
          while (a[i]<x) i++;
          while (x<a[j]) j--;
          if (i<=j) {
             w = a[i]; a[i]=a[j]; a[j]=w; 
             i++;
             j--;
          }
       } while (i<=j);   
       if (j<k) l=i;
       if (k<i) r=j;
    }
    return a[k];
}    

static void MADestD(int n, double *x, double *mean, double *stddev, double *median, double *mad) 
/* estimate median and median of absoluted deviations */
{
    int i;
    double d, a = 0.0, s = 0.0; 
    double *xx; 
    
    xx = malloc(n*sizeof(double));
    memcpy(xx,x,n*sizeof(double));

    *median = qmedD(n,xx);
    for (i=0;i<n;i++) {
	    xx[i] = fabs(x[i]-(*median));
	    d = x[i];
        a += d;
        s += d*d;
    }
    *mean = a/n;
    if ((s-(a*a)/n)>=0) 
    	*stddev = sqrt((s-(a*a)/n)/(n-1));
    else
    	*stddev = 0.0;
    *mad = qmedD(n,xx);
    free(xx);

    /*printf("Median, median of absoluted deviations : %.3f, %.3f\n", *median, *mad); */
}
 
static int am1D(int n, double *x, double *t, double *dt)
/* Newton's iterations */
{
  int i, it;
  double a,b,d,r,s,s2,sum1,sum2,sum3,rp,psir;
  
  if (n<1) {
	*t = 0.0;    /* a few data */
	*dt = -1.0;   
    return 1;   
  }
  
  if (n==1) {  /* only one point, but correct case */
    *t = x[0]; 
    *dt = 0.0;
    return 0; 
  }
  
  /* initial values */
  MADestD(n,x,t,dt,&a,&s);
  
  s = s/0.6745;
  s2 = s*s;

  /* identical points on input */
  if (fabs(s)<epsilon(s)) {                 
      /*a=0.0; for (i=0;i<n;i++) a+=x[i];
      *t = (a/n);
      *dt= 0.0; */
      /*sprintf(buf, "n = %d, t = %.8f, dt = %.8f", n, *t, *dt);
      PrintOut(1, buf);
      for (i=0; i<n; i++) {
      	sprintf(buf, "x[%d] = %.8f", i, x[i]);
      	PrintOut(1, buf);
  	  } */
      return 0;
  }    
  
  /* corrector's estimation */
  for (it=1;it<=maxit;it++) {
       sum1 = 0.0; sum2 = 0.0; sum3 = 0.0;
       for (i=0;i<n;i++) {
          r = (x[i] - a)/s;
          rp = hampel(r);
          sum1 += rp;
          sum2 += dhampel(r);
          sum3 += rp*rp;
          
          //printf("Here1=%f,%f,%f.\n",sum1,sum2,sum3); 
       }
       if (fabs(sum2)<epsilon(sum2)) {
          /*printf("Here2=%.12f,%.12f.\n",fabs(sum2),a); 
          for (i=0;i<n;i++) {
	          r = (x[i] - a)/s;
              printf("x[%d] = %.4f\n", i, x[i]);
              printf("r[%d] = %.4f\n", i, (x[i] - a)/s);
              printf("hampel[%d] = %.4f\n", i, hampel(r));
              printf("dhampel[%d] = %.4f\n", i, dhampel(r));
          }
          printf("----------------\n");*/
          
          ////////////////// POZOR /////////////////////////////
          *t = a; 
          *dt = -1.0;
          ////////////////// POZOR /////////////////////////////
          return 2;
       }
       d = s*sum1/sum2;
       a = a + d;
       b = s2*n/(n-1)*sum3/(sum2*sum2);

       /* exit of iterations: */
       /*printf(stderr,"Prumer, prirustek, chyba: %f, %f, %f, %d\n",a,d,sqrt(b)); */
       if((it>2)&&((d*d<0.01*b)||(fabs(d)<10.0*epsilon(d)))) break;
  }

  /* deviation's estimation */
  sum1 = 0.0; sum3 = 0.0;
  for (i=0;i<n;i++) {
       r = (x[i]-a)/s;
       psir = hampel(r);
       sum1 += psir*psir;
       sum3 += dhampel(r);
  }

  if (sum3==0.0) {
    //printf("Here3=%.12f,%.12f.\n",sum3,a); 
    ////////////////// POZOR /////////////////////////////
    *t = a;
    *dt = -1.0;
    ////////////////// POZOR /////////////////////////////
    return 3;
  }
  
  *t = a;
  *dt = sqrt(s2*n/(n-1)*sum1/(sum3*sum3)*n);
/*  printf("Normal %.3f, %.3f.\n",a,*dt); */
  
  /* Normal return */
  return 0;
}

void cmpack_robustmeanD(int n, double *array, double *median, double *sig)
{
  am1D(n,array,median,sig);
}

void cmpack_fastsky(int nx, int ny, float *ccd, int nmax, float lobad, float hibad, float *skymed, float *skysig)
{
	int i, j;
	int nstep = (int)(sqrt(1.0*nx*ny/nmax)+1);
    float *fbuf = malloc(((nx/nstep)+1)*((ny/nstep)+1)*sizeof(float));
    int l = 0;
    
    for (i=0;i<nx;i+=nstep) {
	    for (j=0;j<ny;j+=nstep) {
		    if ((ccd[i+j*nx]>lobad)&&(ccd[i+j*nx]<hibad)) {
  	            fbuf[l] = ccd[i+j*nx];
	            l++;
            }
        }
    }
   	cmpack_robustmean(l, fbuf, skymed, skysig);
	free(fbuf);
}

void cmpack_fastskyD(int nx, int ny, double *ccd, int nmax, double lobad, double hibad, double *skymed, double *skysig)
{
	int i, j;
	int nstep = (int)(sqrt(1.0*nx*ny/nmax)+1);
    double *fbuf = malloc(((nx/nstep)+1)*((ny/nstep)+1)*sizeof(double));
    int l = 0;
    
    for (i=0;i<nx;i+=nstep) {
	    for (j=0;j<ny;j+=nstep) {
		    if ((ccd[i+j*nx]>lobad)&&(ccd[i+j*nx]<hibad)) {
  	            fbuf[l] = ccd[i+j*nx];
	            l++;
            }
        }
    }
   	cmpack_robustmeanD(l, fbuf, skymed, skysig);
	free(fbuf);
}

void cmpack_fullsky(int nx, int ny, float *ccd, float *skymed, float *skysig)
{
	cmpack_robustmean(nx*ny, ccd, skymed, skysig);
}

void cmpack_fullskyD(int nx, int ny, double *ccd, double *skymed, double *skysig)
{
	cmpack_robustmeanD(nx*ny, ccd, skymed, skysig);
}
