/**************************************************************

trajd.c (C-Munipack project)
Originates from original Munipack by Filip Hroch.

 trajd.f -- translated by f2c (version 19950110).
   You must link the resulting object file with the libraries:
	-lf2c -lm   (in that order)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

**************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>

#include "comfun.h"
#include "trajd.h"

// Converts citizen date to julian date
double cmpack_encodejd(int year, int month, int day, int hour, int minute, int second, int msec)
{
    /* Local variables */
    double jd1;
    int before;
    long d1, d2;

    /* Compute Julian date from input citizen year, month and day. */
    /* Tested for YEAR>0 except 1582-10-07/15 */
    if (year > 1582) {
	    before = 0;
    } else if (year < 1582) {
	    before = 1;
    } else
	if (month > 10) {
	    before = 0;
	} else if (month < 10) {
	    before = 1;
	} else
	if (day >= 15) {
		before = 0;
	} else {
	    before = 1;
	}
    if (month <= 2) {
	    year  -= 1;
	    month += 12;
    }

    d1 = (long)(365.25*year);
    d2 = (long)(30.6001*(month+1));
    jd1 = 1720994.5 + d1 + d2 + day;
    jd1 += 1.0*hour/24;
    jd1 += 1.0*minute/1440.0;
    jd1 += 1.0*second/86400.0;
    jd1 += 1.0*msec/86400000.0;

    if (before) {
	    if (year<0) return jd1-1;
	           else return jd1;
    } else
	    return jd1 + 2 - (year/100) + (year/400);
}

// String to date
int cmpack_strtodate(const char *datestr, int *year, int *month, int *day)
{
	int y, m = 1, d = 1;
	const char *ptr;
	char *endptr;
	
	if (year)  *year  = 0;
	if (month) *month = 0;
	if (day)   *day   = 0;
	
	if (!datestr)
		return 0;
		
	ptr = datestr;
	y = strtol(ptr, &endptr, 10);
	if (y<1600 || y>=2600 || endptr==ptr)
		return 0;
	ptr = endptr + strspn(endptr, "-./: \t,+");
	if (*ptr!='\0') {
		m = strtol(ptr, &endptr, 10); 
		if (m<1 || m>12 || endptr==ptr)
			return 0;
		ptr = endptr + strspn(endptr, "-./: \t,+");
	}
	if (*ptr!='\0') {
		d = strtol(ptr, &endptr, 10); 
		if (d<1 || d>31 || endptr==ptr)
			return 0;
	}
	// Valid date
	if (year)  *year = y;
	if (month) *month = m;
	if (day)   *day = d;
	return 1;
}

// String to time
int cmpack_strtotime(const char *timestr, int *hour, int *minute, int *second, int *msec)
{
	int h = 0, m = 0, x = 0;
	const char *ptr;
	char *endptr;

	if (hour) *hour = 0;
	if (minute) *minute = 0;
	if (second) *second = 0;
	if (msec) *msec = 0;
	
	if (!timestr)
		return 0;

	ptr = timestr;
	if (*ptr!='\0') {
		h = strtol(ptr, &endptr, 10);
		if (h<0 || h>=24 || endptr==ptr)
			return 0;
		ptr = endptr + strspn(endptr, "-./: \t,+");
	}
	if (*ptr!='\0') {
		m = strtol(ptr, &endptr, 10);
		if (m<0 || m>=60 || endptr==ptr)
			return 0;
		ptr = endptr + strspn(endptr, "-./: \t,+");
	}
	if (*ptr!='\0') {
	    x = (int)(1000.0*strtod(ptr, &endptr)+0.5);
	    if (x<0 || x>=60000 || endptr==ptr)
	    	return 0;
    }
    // Valid time
    if (hour)   *hour = h;
    if (minute) *minute = m;
    if (second) *second = x/1000;
    if (msec)   *msec = x%1000;
    return 1;
}

// String to JD
double cmpack_strtojd(const char *datestr, const char *timestr)
{
	int yr, mon, day, hr, min, sec, msec;
	
	if (cmpack_strtodate(datestr, &yr, &mon, &day) &&
		cmpack_strtotime(timestr, &hr, &min, &sec, &msec)) {
        return cmpack_encodejd(yr, mon, day, hr, min, sec, msec);
	}
    return 0;
}

void cmpack_decodejd(double jd, int *year, int *month, int *day, int *hour, int *minute, int *second, int *msec)
{
	/* Local variables */
    long time, z, e, c, a, b, x;

    /* Compute citizen date: year, month and day from input Julian date. */
    /* Only for JD>0! Tested for YEAR>0 except 1582-10-07/15. */
    z = (long)(jd+0.5);
    if (z>=2299163) {
	    long alpha = (long)((z-1867216.25)/36524.25);
	    a = z + 1 + alpha - (alpha/4);
    } else {
	    a = z;
    }
    b = a + 1524;
    c = (long)((b-122.1)/365.25);
    x = (long)(365.25*c);
    e = (long)((b-x)/30.6001);
    *day   = b-x-(int)(30.6001*e);
    *month = (e<=13 ? e-1 : e-13);
    *year  = (*month>=3 ? c-4716 : c-4715);

    // Cas v milisekundach
    time = (long)((jd+0.5-z)*86400000.0+0.5);
    *hour   = (time/3600000);
    *minute = (time/60000)%60;
    *second = (time/1000)%60;
    *msec   = time%1000;
}

// Date to string
void cmpack_datetostr(int year, int month, int day, char *datestr)
{
	if (datestr)
		sprintf(datestr, "%04d-%02d-%02d", year, month, day);
}

// Time to string
void cmpack_timetostr(int hour, int minute, int second, int msec, char *timestr)
{
	if (timestr)
		sprintf(timestr, "%02d:%02d:%02d.%03d", hour, minute, second, msec);
}

void cmpack_jdtostr(double jd, char *datestr, char *timestr)
{
   int yr, mon, day, hr, min, sec, msec;
   cmpack_decodejd(jd, &yr, &mon, &day, &hr, &min, &sec, &msec);
   cmpack_datetostr(yr, mon, day, datestr);
   cmpack_timetostr(hr, min, sec, msec, timestr);
}

double cmpack_siderealtime(double jd)
// Computes Greenwich mean sidereal time in hours
{
	double jd0, t3, s0, s1, x;

	// JD at 0h UT
	jd0 = floor(jd) + 0.5;
	// Julian centuries
	t3 = (jd0 - 2451545.0)/36525.0;
	// Mean sidereal time for 0h UT
	s0 = 6.697374558 + 2400.05133691*t3 + 0.0000258622*t3*t3 - 0.0000000017*t3*t3*t3;
	// Mean sidereal time
	s1 = s0/24.0 + 1.0027379093*(jd-jd0);
	x = 24.0*(s1 - floor(s1));
	printf("Sidereal time: %.5f", x);
	return x;
}

double cmpack_airmass(double jd, double ra, double dec, double lon, double lat)
// Compute airmass coefficient X. ra is in hours; dec, lon, lat in degrees
{
	double lmst, har, sinh, secz, x;

	// Local mean sidereal time
	lmst = cmpack_siderealtime(jd) + (lon/15.0);
	// Hour angle
	har = (lmst - ra)/12.0*PI;
	// sec of zenital distance
	lat = lat/180.0*PI;
	dec = dec/180.0*PI;
	sinh = sin(dec)*sin(lat) + cos(dec)*cos(har)*cos(lat);
	// Is the star above the horizon?
	if (sinh>=1.0e-4) {
		secz = 1.0/sinh;
		// Air mass coefficient
		x = secz - 0.0018167*(secz-1) - 0.002875*(secz-1)*(secz-1) - 0.0008083*(secz-1)*(secz-1)*(secz-1);
		if (x>=1.0)
			return x;
	}
	return -1.0;
}

void cmpack_azalt(double jd, double ra, double dec, double lon, double lat, double *az, double *alt)
// Transforms equatorial coordinates to horizontal system 
{
	double lmst, har, x, y;
	
	// Local mean sidereal time (in hours)
	lmst = cmpack_siderealtime(jd) + (lon/15.0);
	// Hour angle (in radians)
	har = (lmst-ra)/12.0*PI;
    dec = dec/180.0*PI;
	lat = lat/180.0*PI;
    // Compute azimuth
    x = cos(dec)*sin(har);
    y = cos(dec)*cos(har)*sin(lat) - sin(dec)*cos(lat);
  	x = 0.75 - (atan2(y, x)/(2.0*PI));
   	if (az) *az = 360.0*(x-floor(x));
	// Compute altitude
    if (alt) *alt = asin(sin(dec)*sin(lat) + cos(dec)*cos(har)*cos(lat))/PI*180.0;
}

void cmpack_rdtolb(double ra, double de, double *la, double *be)
/* Prevede souradnice ra,de (rovnikove) na la, be (ekliptikalni) */
{
  /* Prevod na radiany */
  ra = ra/12.0*PI;
  de = de/180.0*PI;
  /* Do intervalu 0..2pi */
  while (ra<0.0)   ra += 2*PI;
  while (ra>=2*PI) ra -= 2*PI;
  /* Vylouceni singularit */
  if (de>PI/2-TOL) {
    *la=PI/2;
    *be=PI/2-EPS;
  } else
  if (de<-PI/2+TOL) {
    *la=3*PI/2;
    *be=EPS-PI/2;
  } else
  if (fabs(ra-PI/2)<TOL) {
    *la=PI/2;
    *be=de-EPS;
  } else
  if (fabs(ra-3*PI/2)<TOL) {
    *la=3*PI/2;
    *be=de+EPS;
  } else {
    /* Normalni vypocet */
    *be=asin(cos(EPS)*sin(de)-sin(EPS)*cos(de)*sin(ra));
    *la=atan((sin(EPS)*sin(de)+cos(EPS)*cos(de)*sin(ra))/(cos(de)*cos(ra)));
    /* Do spravneho kvadrantu */
    if (cos(ra)<0.0) *la += PI;
  }
}

void cmpack_sun(double jd, double *sun_ls, double *sun_rs)
/* Vypocet ekl. delky slunce a vzdalenosti zeme - slunce */
{
 double t,vt,ls,ms,m5,m4,m2,lm;

 t =jd-2451545.0;
 vt=1+(t/36525.0);
 ls=(0.779072 + 0.00273790931*t)*2*PI;
 ms=(0.993126 + 0.00273777850*t)*2*PI;
 m5=(0.056531 + 0.00023080893*t)*2*PI;
 m4=(0.053856 + 0.00145561327*t)*2*PI;
 m2=(0.140023 + 0.00445036173*t)*2*PI;
 lm=(0.606434 + 0.03660110129*t)*2*PI;
 ls=ls*3600*180/PI+6910*sin(ms)
       +  72*sin(2*ms)
       -17*vt*sin(ms)
       -   7*cos(ms-m5)
       +   6*sin(lm-ls)
       +   5*sin(4*ms+8*m4+3*m5)
       -   5*cos(2*ms-2*m2)
       -   4*sin(ms-m2)
       +   4*cos(4*ms-8*m4+3*m5)
       +   3*sin(2*ms-2*m2)
       -   3*sin(m5)
       -   3*sin(2*ms-2*m5);
 /* Ekl. delka slunce v radianech */
 *sun_ls=ls/3600/180*PI;
 /* vzdalenost slunce v AU */
 *sun_rs=1.00014-0.01675*cos(ms)-0.00014*cos(2*ms);
}

double cmpack_hcor(double la, double be, double ls, double rs)
/* Pocita heliocentrickou korekci */
{
  return -0.00577552*rs*cos(be)*cos(la-ls);
}

