/**************************************************************

hcor.c (C-Munipack project)
Copyright (C) 2003 David Motl, dmotl@volny.cz

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

**************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>

#include "comfun.h"
#include "convert.h"
#include "hcor.h"

static int    hkor_set = 0;
static int    hkor_mode = 0;
static double hkor_ra = 0.0;
static double hkor_de = 0.0;
static double hkor_la = 0.0;
static double hkor_be = 0.0;
static double hkor_ls = 0.0;
static double hkor_rs = 0.0;

int helcor_initd(double ra, double de, int mode)
/* Prevede souradnice ra,de (rovnikove) na la, be (ekliptikalni) */
{
  hkor_ra = ra;
  hkor_de = de;
  cmpack_rdtolb(hkor_ra, hkor_de, &hkor_la, &hkor_be);
  hkor_mode = mode;
  hkor_set = 1;      /* The parameters was set */
  return 0;
}

int helcor_inits(const char *ra, const char *de, int mode)
{
 double a, d;
 int stat;

 if (ra) {
	 stat = cmpack_strtora(ra, &a);
	 if (stat!=0) return stat;
 } else {
	 PrintOut(0, "Missing R.A. of object");
	 return MPK_INVALID_PAR;
 }
 if (de) {
	 stat = cmpack_strtodec(de, &d);
	 if (stat!=0) return stat;
 } else {
	 PrintOut(0, "Missing DEC. of object");
	 return MPK_INVALID_PAR;
 }
 return helcor_initd(a,d,mode);
}

int helcor_gtoh(double jdgeo, double *jdhel)
{
 cmpack_sun(jdgeo, &hkor_ls, &hkor_rs);
 *jdhel = jdgeo + cmpack_hcor(hkor_la, hkor_be, hkor_ls, hkor_rs);
 return 0;
}

int helcor_htog(double jdhel, double *jdgeo)
{
 cmpack_sun(jdhel, &hkor_ls, &hkor_rs);
 *jdgeo = jdhel - cmpack_hcor(hkor_la, hkor_be, hkor_ls, hkor_rs);
 return 0;
}

int helcor_hcor(double jd, double *helcor)
{
 cmpack_sun(jd, &hkor_ls, &hkor_rs);
 *helcor = cmpack_hcor(hkor_la, hkor_be, hkor_ls, hkor_rs);
 return 0;
}

int helcor(const char *infile, const char *outfile)
{
	FILE  *f, *g;
	char   buf[MAXLINE], msg[MAXLINE], eol[MAXLINE], *endptr, *aux;
	double jdin, jdout, hcor;
	int    i, dec, rev, inc, mod, head_found, jd_short;
	
	sprintf(buf,"%s -> %s",infile,outfile);
	PrintOut(0, buf);
	rev = (hkor_mode & 0x01)!=0;	// Input is 0=GJD, 1=HJD
	mod = (hkor_mode & 0x02)==0;	// Output is 0=Same as input, 1=Modified
	inc = (hkor_mode & 0x04)!=0;	// Corr 0=don't include, 1=include
	
	f = fopen(infile,"r");
	if (f==NULL)
		return MPK_CANNOT_OPEN_SRC;
	
	g = fopen(outfile,"w");
	if (g==NULL) 
		return MPK_CANNOT_OPEN_OUT;

	head_found = 0;
	while (fgets(buf,MAXLINE,f)!=NULL) {
		if (!head_found) {
      		if (strstr(buf, "JD ")==buf || strstr(buf, "JDHEL ")==buf || strstr(buf, "JDGEO ")==buf) {
				// This is a header line 
	      		if (!mod) {
					if (inc) {
						// Strip out but preserve any CRLF characters
						i = strcspn(buf, "\r\n");
						strcpy(eol, buf+i);
						buf[i] = '\0';
						fprintf(g,"%s HELCOR%s", buf, eol);
					} else {
						fprintf(g,"%s", buf);
					}
	      		} else {
		      		endptr = buf + strcspn(buf, " ");
		      		aux = (!rev ? "JDHEL" : "JDGEO");
					if (inc) {
						// Strip out but preserve any CRLF characters
						i = strcspn(buf, "\r\n");
						strcpy(eol, buf+i);
						buf[i] = '\0';
						fprintf(g,"%s%s HELCOR%s", aux, endptr, eol);
					} else {
						fprintf(g,"%s%s", aux, endptr);
					}
				}
				head_found = 1;
				continue;
			}
		}
		
		jdin = strtod(buf,&endptr);
		if (endptr==buf) {
			// Any other line
			fprintf(g,"%s",buf);
			continue;
		}
			
		// Determine, how many decimal places were used 
		aux = strchr(buf,'.');
		if (aux<endptr) dec = (int)(endptr-aux)-1;
		else 			dec = DECIMALS;
		
		if ((jdin>10000 && jdin<99999) || (jdin>2400000 && jdin<3000000)) {
			jd_short = jdin<100000;
			// Short JD
			if (jd_short) 
			  jdin += 2400000.0;
			cmpack_sun(jdin, &hkor_ls, &hkor_rs);
			hcor = cmpack_hcor(hkor_la, hkor_be, hkor_ls, hkor_rs);
			if (!mod) {
				jdout = jdin;
				sprintf(msg, "%.*f -> %.*f", dec, jdin, dec, hcor);
			} else				
			if (rev) {
				jdout = jdin - hcor;
				sprintf(msg, "%.*f - (%.*f) = %.*f", dec, jdin, dec, hcor, dec, jdout);
			} else {
				jdout = jdin + hcor;
				sprintf(msg, "%.*f + (%.*f) = %.*f", dec, jdin, dec, hcor, dec, jdout);
			}
			PrintOut(1, msg);
			if (jd_short) 
			  jdout -= 2400000.0;
			if (!mod) {
				if (inc) {
					i = strcspn(buf, "\r\n");
					strcpy(eol, buf+i);
					buf[i] = '\0';
					fprintf(g,"%s %.*f%s", buf, dec, hcor, eol);
				} else {
					fprintf(g, "%s" , buf);
				}
			} else {
				if (inc) {
					i = strcspn(buf, "\r\n");
					strcpy(eol, buf+i);
					buf[i] = '\0';
					fprintf(g,"%.*f%s %.*f%s", dec, jdout, endptr, dec, hcor, eol);
				} else {
					fprintf(g, "%.*f%s" , dec, jdout, endptr);
				}
			}
		} else {
			// Invalid line
			fprintf(g,"%s",buf);
		}
	}  
	fclose(f);
	fclose(g);

	return 0;
}

int helcor_clean(void)
{
	hkor_set = 0;
 	return 0;
}
