/**************************************************************

hterm.c (C-Munipack project)
Copyright (C) 2003 David Motl, dmotl@volny.cz

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

**************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "comfun.h"
#include "convert.h"
#include "cmdparser.h"
#include "hterm.h"
#include "hcor.h"

static int    mode = 0;
static double ra  = 0.0;
static double dec = 0.0;

void PrintCoords(int mode);
void PrintTHelp();

int hcor_term(int reverse, const char *strra, const char *strdec)
{
	char *argv[64], *prompt, outf[MAXLINE], *endptr;
	char datestr[32], timestr[32];
	int argc, y, sgn;
	double jdin, hcor, la, be, ls, rs, x;

	printf("This is terminal mode. Enter 'h' command for help or 'q' for exit.\n\n");
	cmpack_setoutput(NULL, MPK_LEVEL_QUIET); 	// disable outputs

	mode = 0;
 	if (cmpack_strtora(strra, &x)==0) {
	 	ra = x;
 		mode |= 0x01;
	}
	if (cmpack_strtodec(strdec, &x)==0) {
		dec = x;
		mode |= 0x02;
	}

	PrintCoords(mode);

    while (mode>=0) {
	    if ((mode & 0x01)==0) prompt = "R.A.:";
	    else if ((mode & 0x02)==0) prompt = "DEC.:";
	    else prompt = (reverse ? "JDhel:" : "JDgeo:");

	    getstr(prompt, outf, MAXLINE);
	    argc = parse_args(argv, outf);
	    if (argc==0) {
		    // No command => Print coordinates
		    PrintCoords(mode);
		    continue;
	    }
  		if (strncmp("q", argv[0], strlen(argv[0])) == 0) {
	  		// Quit command
	    	mode = -1;
	        break;
  		}
  		if (strncmp("h", argv[0], strlen(argv[0])) == 0
  		   || strncmp("?", argv[0], strlen(argv[0])) == 0
		   || strncmp("help", argv[0], strlen(argv[0])) == 0) {
	  		// Print help
	    	PrintTHelp();
			continue;
  		}
  		if (strncmp("r", argv[0], strlen(argv[0])) == 0 || 
			strncmp("R", argv[0], strlen(argv[0])) == 0) {
	  		// Set reverse mode
	  		reverse = 1;
	  		printf("Mode set to reverse (jdhel --> jdgeo).\n");
	  		continue;
		    }
  		if (strncmp("n", argv[0], strlen(argv[0])) == 0) {
	  		// Set normal mode
	  		reverse = 0;
	  		printf("Mode set to normal (jdgeo --> jdhel).\n");
	  		continue;
  		}
  		if (strncmp("ra", argv[0], strlen(argv[0]))==0) {
	  		if (argc==1) {
		  		// Print R.A.
		  		PrintCoords(0x01);
	  		} else {
		  		// Set R.A.
		  	    if (cmpack_strtora(argv[1], &x)==0) {
			  	    ra = x;
			  	    PrintCoords(0x01);
			  	    mode |= 0x01;
		  	    } else {
			  	    PrintError(PROGNAME, NULL, 0, MPK_INVALID_OBJRA);
		  	    }
	  		}
	  		continue;
  		}
  		if (strncmp("dec", argv[0], strlen(argv[0]))==0) {
	  		if (argc==1) {
		  		// Print DEC.
		  		PrintCoords(0x02);
	  		} else {
		  		// Set DEC.
		  	    if (cmpack_strtodec(argv[1], &x)==0) {
			  	    dec = x;
			  	    PrintCoords(0x02);
			  	    mode |= 0x02;
		  	    } else {
			  	    PrintError(PROGNAME, NULL, 0, MPK_INVALID_OBJDEC);
		  	    }
	  		}
	  		continue;
  		}
  		
  		if ((mode & 0x01)==0) {
	  		// Set R.A.
	  	    if (cmpack_strtora(outf, &x)==0) {
		  	    ra = x;
		  	    PrintCoords(0x01);
		  	    mode |= 0x01;
	  	    } else {
		  	    PrintError(PROGNAME, NULL, 0, MPK_INVALID_OBJRA);
	  	    }
	  		continue;
  		}
  		if ((mode & 0x02)==0) {
	  		// Set DEC.
	  	    if (cmpack_strtodec(outf, &x)==0) {
		  	    dec = x;
		  	    PrintCoords(0x02);
		  	    mode |= 0x02;
	  	    } else {
		  	    PrintError(PROGNAME, NULL, 0, MPK_INVALID_OBJDEC);
	  	    }
	  		continue;
  		}
  		
  		jdin = strtod(outf,&endptr);
  		if (endptr!=outf && ((jdin>=10000 && jdin<100000) || (jdin>=2400000 && jdin<3000000))) {
	  		if ((mode & 0x03)!=0x03) {
	  			PrintError(PROGNAME, NULL, 0, MPK_MISSING_OBJCOORDS);
	  			continue;
  			}
  			if (jdin<100000)
	      		jdin += 2400000.0;
	      	printf("------------------------------------------------\n");
			cmpack_rdtolb(ra, dec, &la, &be);
		 	cmpack_sun(jdin, &ls, &rs);
 			hcor = cmpack_hcor(la, be, ls, rs);
 			if (hcor>-1.0 && hcor<0.0) {
	 			sgn = -1;
	 			y = (int)(-hcor*24.0*3600000.0+0.5);
 			} else
 			if (hcor>=0.0 && hcor<1.0) {
	 			sgn = +1;
	 			y = (int)(hcor*24.0*3600000.0+0.5);
 			} else {
	 			sgn = y = 0;
 			}
 			if (!reverse) {
	 			double jdhel = jdin + hcor;
	      		cmpack_jdtostr(jdin, datestr, timestr);
	      		printf("JD (geoc.)  : %.5f\n", jdin);
	      		printf("Date & time : %s %s UT\n", datestr, timestr);
	    	  	printf("Hel. corr.  : %.5f d = %d min %.2d.%.3d s \n", hcor, sgn*(y/60000), ((y/1000)%60), (y%1000));
	      		cmpack_jdtostr(jdhel, datestr, timestr);
	      		printf("JD (hel.)   : %.5f\n", jdhel);
	      		printf("Date & time : %s %s UT\n", datestr, timestr);
  			} else {
	  			double jdgeo = jdin - hcor;
	      		cmpack_jdtostr(jdin, datestr, timestr);
	      		printf("JD (hel.)   : %.5f\n", jdin);
	      		printf("Date & time : %s %s UT\n", datestr, timestr);
	    	  	printf("Hel. corr.  : %.5f d = %d min %.2d.%.3d s \n", hcor, sgn*(y/60000), ((y/1000)%60), (y%1000));
	      		cmpack_jdtostr(jdgeo, datestr, timestr);
	      		printf("JD (geoc.)  : %.5f\n", jdgeo);
	      		printf("Date & time : %s %s UT\n", datestr, timestr);
  			}
	      	printf("------------------------------------------------\n");
      		continue;
      }
      printf("Invalid julian date.\n");
  }
  printf("\nExiting terminal mode.\n");

  /* Normal return */
  return 0;
}

void PrintCoords(int mode)
/* Prints objects coordinates */
{
	char buf[32];
	
    if ((mode & 0x01)!=0) {
	  	cmpack_ratostr(ra, buf);
	  	printf("R.A. = %s\n", buf);
  	}
    if ((mode & 0x02)!=0) {
	  	cmpack_dectostr(dec, buf);
	  	printf("DEC. = %s\n", buf);
  	}
  	if (mode==0) {
		printf("Object's coordinates have not been set yet.\n");
	}
}

void PrintTHelp()
/* Print help */
{
 printf("HelCor (C-Munipack) %s\n",VERSION);
 printf(COPYRIGHTLINE);
 printf(WARRANTYLINE);
 printf("\nCommands of terminal mode:\n");
 printf("    r            -- Set reverse mode (jdhel -> jdgeo)\n");
 printf("    n            -- Set normal mode (jdgeo -> jdhel)\n");
 printf("    9999999.999  -- Julian date to convert\n");
 printf("    q            -- exit program\n");
 printf("    empty line   -- print current settings\n");
 printf("    h, ? or help -- print this help\n");
 printf("    ra hh:mm     -- Set R.A. of object\n");
 printf("    dec +dd:mm   -- Set DEC. of object\n");
 printf("\n");
 PrintCoords(mode);
 printf("\n");
 printf(HOMEPAGELINE);
 printf(BUGREPORTS);
 printf("\n");
}
