/**************************************************************

matfun.c (C-Munipack project)
Copyright (C) 2003 David Motl, dmotl@volny.cz

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

**************************************************************/

#define _NO_OLDNAMES
#include <math.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <errno.h> 

#include "comfun.h"
#include "match.h"
#include "matfun.h"

struct StackType               
{
  int    matched;              /* Kolikrat stejna serie */
  int    *id1;           		/* Cisla hvezd v serii 1 */
  int    *id2;           		/* Cisla hvezd v serii 2 */
  double m[9];                 /* Tranformation matrix */
  double sumsq;                /* Sum of squares */
  double tolerance;			   /* Tolerance */
  int    nstar, mstar;         /* Polygon vertices, matched stars */
  struct StackType *next;      /* dalsi hvezda nebo NULL */
};

typedef struct StackType StackType;

/* Reference file */
int    mmatch_c1;                           /* No. of stars in input file */
int    mmatch_n1;                           /* No. of stars for matching */
double mmatch_x1[MAXSTARS], mmatch_y1[MAXSTARS];   /* Coordinates of stars */

/* Input file */
int    mmatch_c2;                           /* No. of stars in input file */
int    mmatch_n2;                           /* No. of stars for matching */
double mmatch_x2[MAXSTARS], mmatch_y2[MAXSTARS];   /* Coordinates of stars */
char   *mmatch_line1, *mmatch_line2;               /* Header lines of input file */
char   *mmatch_l1[MAXSTARS], *mmatch_l2[MAXSTARS]; /* Lines of stars in input file */
int    mmatch_id[MAXSTARS];					/* Cross-reference id */
double mmatch_offx, mmatch_offy;			/* Frame offset in pixels */

/* Parameters */
int    mmatch_nstar;                    /* Number of apices of polygons */
int    mmatch_maxstar;                  /* Maximum number of stars used in Solve procedure */
double mmatch_clip;                     /* Clipping factor in sigmas */
int    mmatch_spfields;					/* Matching mode */
double mmatch_maxoffset;				/* Max. position offset for Simple method */

/* Temporary variables */
int    *mmatch_idr1, *mmatch_idr2;
int    *mmatch_id1, *mmatch_id2;
double *mmatch_r1, *mmatch_r2;
double *mmatch_k2, *mmatch_dev;
double mmatch_z1, mmatch_z2;

/* Stack of matrices found */
StackType *mmatch_first = NULL; 
StackType *mmatch_last = NULL;

void Center(int n, double *x, double *y, double t[2])
/* Calcualtion of the mean (t) of the coordinates (x,y) */
{
	int i; 
	t[0] = 0.0; 
	t[1] = 0.0;
	for (i=0;i<n;i++) {
		t[0] += x[i]; 
		t[1] += y[i]; 
	}
	t[0] /= n; 
	t[1] /= n; 
}

double Uhel(double a[2], double b[2])
/* Arc between vectors a and b */
{
	double c, a2, b2, x; 

	c = (a[0]*b[1]-a[1]*b[0]); 
    a2 = a[0]*a[0]+a[1]*a[1];
    b2 = b[0]*b[0]+b[1]*b[1];
    if (a2*b2==0.0) {
	    return -1.0;
    } else {
	    x = sqrt((c*c)/(a2*b2));
	    if (fabs(x)>1.0) x = 1.0;
	    return asin(x);
    }
}

void Serad(double a[3])
/* Sort array A from the greatest to the smallest. */
{
	double t; 
	int i, j; 
	
	for (i=1;i<=2;i++) {
		for (j=2;j>=i;j--) {
			if (a[j-1]<a[j]) {
				t = a[j-1]; 
				a[j-1] = a[j]; 
				a[j] = t; 
			}
		}
	}
}	

void UV(int l, int j, int k, double *x, double *y, double *u, double *v)
/* Calculation of the triangle coordinates (u,v) */
{
	double d[3];
	
    d[0] = (x[l]-x[j])*(x[l]-x[j])+(y[l]-y[j])*(y[l]-y[j]);
    d[1] = (x[l]-x[k])*(x[l]-x[k])+(y[l]-y[k])*(y[l]-y[k]);
    d[2] = (x[k]-x[j])*(x[k]-x[j])+(y[k]-y[j])*(y[k]-y[j]);
    Serad(d);
    
    //fprintf(stderr,"UV: %.3f %.3f %.3f ",d[0],d[1],d[2]);
    
    *u = sqrt(d[1]/d[0]);
    *v = sqrt(d[2]/d[0]);

    //fprintf(stderr," => %.3f %.3f\n",*u,*v);
}
 
void Trafo(double *m, double x1, double y1, double *x2, double *y2)
/* Transform coordinates: (x2,y2,1) = M(x1,mmatch_y1,1) */
{
	*x2 = M(0,0)*x1+M(0,1)*y1+M(0,2); 
	*y2 = M(1,0)*x1+M(1,1)*y1+M(1,2);
}	

void StInit(void)
/* Initializes the stack */
{
	mmatch_first = NULL;
	mmatch_last  = NULL;
	mmatch_n2 = 0;
	mmatch_c2 = 0;
	mmatch_line1 = NULL;
	mmatch_line2 = NULL;
	memset(mmatch_l1, 0, MAXSTARS*sizeof(char*));
	memset(mmatch_l2, 0, MAXSTARS*sizeof(char*));
}

void StAdd(int nstar, double sumsq, double tolerance, int mstar, double *m)
/* Write next record or increased found serie */
{
	StackType *it, *ptr;
	int je1, j, ll;
	
    /* Finding the serie in known series */
    for (ptr=mmatch_first; ptr!=NULL; ptr = ptr->next) {
	    if (ptr->nstar==nstar) {
		    je1 = 0; 
		    for (j=0;j<nstar;j++) {
			    for (ll=0;ll<nstar;ll++) {
				    if ((mmatch_id1[ll]==ptr->id1[j]) && (mmatch_id2[ll]==ptr->id2[j])) {
					    je1++; 
					    break;
				    }
			    }
		    }
	        if (je1==nstar) {
		        ptr->matched++;
	            break; 
	        }
        }
    }
    	
    /* Add a new transformation matrix */
    if (ptr==NULL) {
	    it = malloc(sizeof(StackType));
	    it->sumsq = sumsq; 
	    it->tolerance = tolerance;
	    it->matched = 1;
	    it->mstar = mstar;
	    it->nstar = nstar;
        memcpy(it->m,m,9*sizeof(double));
        it->id1 = malloc(nstar*sizeof(int));
        memcpy(it->id1,mmatch_id1,nstar*sizeof(int));
        it->id2 = malloc(nstar*sizeof(int));
        memcpy(it->id2,mmatch_id2,nstar*sizeof(int));
        it->next = NULL;
        if (mmatch_last) {
	        mmatch_last->next = it; 
            mmatch_last = it;
        } else {
	        mmatch_last = mmatch_first = it;
        }	        
    }
}

void StDump(void)
/* Dump content of the stack to the debug output */
{
	StackType	*ptr;
	char 		msg[256];
	double 		m[9];
	
    PrintOut(1, "NStar MStar Match SumSq      Tolerance Matrix");
    for (ptr=mmatch_first; ptr!=NULL; ptr=ptr->next) {
	    memcpy(m, ptr->m, 9*sizeof(double));
	    sprintf(msg, "%5d %5d %5d %10.5f %10.1f %.3f %.3f %.3f %.3f %.3f %.3f", ptr->nstar, ptr->mstar,
	    	ptr->matched, ptr->sumsq, ptr->tolerance, M(0,0), M(0,1), M(0,2), M(1,0), M(1,1), M(1,2));
	    PrintOut(1, msg);
    }   
}	


void StSelect(int *mstar, double *sumsq, double *tolerance, double *matrix)
/* Find the most numerous series and calculation of result transformation */
{
	int j, n;
	StackType *best, *ptr;
	
	/* Find the best case ('matched' values are compared) */
	j = n = 0; best = NULL;
    for (ptr=mmatch_first; ptr!=NULL; ptr=ptr->next) {
	    if ((ptr->nstar>n) || (ptr->nstar==n && ptr->matched>j)) {
		    j = ptr->matched;
		    n = ptr->nstar;
		    best = ptr; 
	    }
    }
	if (best) {
		/* Found */
		*mstar = best->mstar;
		*sumsq = best->sumsq;
		*tolerance = best->tolerance;
		memcpy(matrix, best->m, 9*sizeof(double));
    } else {
        /* Not found */
        *mstar = 0; 
        *sumsq = 0.0;
        *tolerance = 0.0;
        memset(matrix, 0, 9*sizeof(double));
    }        
}

void StClear(void)
/* Clears the stack */
{
	StackType *ptr, *next; 
	int i; 
	
	if (mmatch_first) {
		ptr = mmatch_first;
		while (ptr) {
			next = ptr -> next;
			if (ptr->id1)
				free(ptr->id1);
			if (ptr->id2)
				free(ptr->id2);
			free(ptr);
			ptr = next;
		}
	}
	mmatch_first = NULL;
	mmatch_last  = NULL;
 
	if (mmatch_line1) {
		free(mmatch_line1);
		mmatch_line1 = NULL;
	}
 	if (mmatch_line2) {
 		free(mmatch_line2);
 		mmatch_line2 = NULL;
	}
 	for (i=0;i<MAXSTARS;i++) { 
		if (mmatch_l1[i]) 
			free(mmatch_l1[i]);
		if (mmatch_l2[i]) 
			free(mmatch_l2[i]); 
 	}
 	mmatch_n2 = 0;
 	mmatch_c2 = 0;
	memset(mmatch_l1, 0, MAXSTARS*sizeof(char*));
	memset(mmatch_l2, 0, MAXSTARS*sizeof(char*));
}
