/**************************************************************

tcor.c (C-Munipack project)
Copyright (C) 2003 David Motl, dmotl@volny.cz

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

**************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fitsio.h>

#include "comfun.h"
#include "fitsfun.h"
#include "tcor.h"

double tcor_tcor = 0.0;    /* in seconds */

int tcor_init(double tcor)
/* initialization of global variables */
{
 char buf[MAXLINE];

 tcor_tcor = tcor;
 sprintf(buf,"Time correction: %.3lf s",tcor_tcor);
 PrintOut(0, buf);
 return 0;
}

int tcor(const char *sci, const char *out, char *date, char *time)
/* Do time correction */
{
 double jd;
 int  sstat = 0, ostat = 0;
 char buf[MAXLINE];
 fitsfile *sfits;
 fitsfile *ofits;

 if (out) sprintf(buf,"(%s) => (%s)",sci,out);
     else sprintf(buf,"(%s)",sci);
 PrintOut(0, buf);

 /* Open source file */
 ffdkopn(&sfits,sci,(out?READONLY:READWRITE),&sstat);
 if (sstat) {
     PrintOut(1, "Source file does not exists or has invalid format.");
     return MPK_CANNOT_OPEN_SRC;
 }

 /* Read date and time */
 sstat = fits_getjd(sfits,&jd);
 if (sstat!=0) {
	 ffclos(sfits,&sstat);
     return MPK_INVALID_HEADER;
 }

 if (out) {
     /* Open destination file */
     remove(out);
     ffdkinit(&ofits,out,&ostat);
     if (ostat!=0) {
	     ffclos(sfits,&sstat);
	     return MPK_CANNOT_OPEN_OUT;
     }
     /* Copy data from source to destination */
     ffcopy(sfits,ofits,0,&sstat);
     if (sstat!=0) {
	     ffclos(ofits,&ostat);
	     ffclos(sfits,&sstat);
	     return sstat;
     }
     /* Set new date and time */
     ostat = fits_setjd(ofits,jd+(tcor_tcor/86400.0));
 	 if (ostat) {
	 	 ffclos(ofits,&ostat);
	     ffclos(sfits,&sstat);
	 	 return ostat;
 	 }
     ffphis(ofits,"Time corrected.",&ostat);
     if ((date)&&(time))
	     fits_getdatetime(ofits,date,time);
     /* Close destination file */
   	 ffclos(ofits,&ostat);
 } else {
	 /* Write new date and time to file */
     sstat = fits_setjd(sfits,jd+(tcor_tcor/86400.0));
     ffphis(sfits,"Time corrected.",&sstat);
     if ((date)&&(time))
	     fits_getdatetime(sfits,date,time);
 }

 /* Close source file */
 ffclos(sfits,&sstat);
 if (sstat!=0) return sstat;

 /* Normal return */
 return 0;
}

int tcor_clean(void)
{
 return 0;
}
